import { Controller, Get, Header, Param, ParseUUIDPipe, Res } from '@nestjs/common';
import { Response } from 'express';

import { EventAttendance, EventAttendee } from '../events/attendee.entity';
import { Event } from '../events/event.entity';
import { EventsService } from '../events/events.service';
import { UserService } from '../user/user.service';

import { VCalendar } from './VCalendar';

@Controller('calendars')
export class CalendarsController {

  constructor(
    private userService: UserService,
    private eventService: EventsService) { }

  /**
   * creates a calendar out of the user events
   * @TODO look for possibility to not us Res
   * @param uuid
   * @param res
   */
  @Get(':uuid')
  @Header('Content-Type', 'text/calendar;charset=utf-8')
  @Header('Content-Disposition', 'attachment; filename="eventing.calendar.my.ics"')
  public async one(@Param('uuid', ParseUUIDPipe) uuid: string, @Res() res: Response) {
    const user = await this.userService.one({ publicId: uuid });
    const events = await this.eventService.repository.manager.createQueryBuilder(Event, 'event')
      .select()
      .innerJoinAndSelect(EventAttendee, 'rsvp', 'event.id = rsvp.eventId')
      .where('rsvp.userId = :userId', { userId: user.id })
      .andWhere('rsvp.status != :status', { status: EventAttendance.declined })
      .andWhere('event.dateStart >= :dateStart', { dateStart: Date.now() / 1000 })
      .getMany();

    const calendar: VCalendar = new VCalendar('Eventing', null, null, null, `-//Eventing//${user.name}//EN`);
    calendar.createCalendarFromEvents(events);
    res.send(calendar.toICSString());
  }
}
