import { Injectable } from '@nestjs/common';
import { ActivityTypes, BotFrameworkAdapter, ChannelAccount, ConversationReference, ConversationState, TurnContext, UserState } from 'botbuilder';

@Injectable()
export class ChatbotService {

    public readonly conversations: Map<string, Partial<ConversationReference>> = new Map();

    constructor(
        public readonly adapter: BotFrameworkAdapter,
        public readonly conversationState: ConversationState,
        public readonly userState: UserState) { }

    public async continueConversation(userId: string, logic: (context: TurnContext) => Promise<void>): Promise<void> {
        if (!this.conversations.has(userId)) {
            throw new Error('User not found');
        }

        await this.adapter.continueConversation(this.conversations.get(userId), async (context) => {
            context.activity.type = ActivityTypes.Message;
            await logic(context);
        });
    }

    public addConversationReference(context: TurnContext, member: ChannelAccount) {
        const conversationReference = TurnContext.getConversationReference(context.activity);
        this.conversations.set(member.id, conversationReference);
    }
}
