import { MessageFactory } from 'botbuilder';
import { ComponentDialog, DialogContext, TextPrompt, WaterfallDialog } from 'botbuilder-dialogs';

import { isNoReply } from '../../utility/UserInputParser';

export namespace HelpDialog {
    export interface DialogOptions {
        text?: string;
        cta?: string;
    }
}

const helpMsg = `Hallo, ich bin der EventingBot!

Du kannst mit mir folgendes machen:

- /login dich auf einem neuen Gerät einloggen, um alles mögliche online zu verwalten

- /cal den Link zu deinem persönlichen Kalender anzeigen lassen`;

export class HelpDialog extends ComponentDialog<HelpDialog.DialogOptions> {

    constructor(readonly id: string) {
        super(id);
        this.addDialog(
            new WaterfallDialog<HelpDialog.DialogOptions>(id + 'wf', [
                async (step) => {
                    const reply = MessageFactory.suggestedActions([step.options.cta], step.options.text);
                    return await step.prompt(this.id + 'tp', reply);
                },
                async (step) => {
                    if (step?.result === step.options.cta) {
                        await step.context.sendActivity(MessageFactory.text(helpMsg));
                        step.context.activity.text = '';
                    } else if (isNoReply(step?.result)) {
                        // skip no help needed replies
                        step.context.activity.text = '';
                    }

                    return step.endDialog();
                }
            ])
        );
        this.addDialog(new TextPrompt(id + 'tp'));
    }

    public async onBeginDialog(innerDC: DialogContext, options?: HelpDialog.DialogOptions) {
        return await super.onBeginDialog(innerDC, {
            cta: options?.cta || 'Ja, gerne',
            text: options?.text || 'Kann ich dir noch weiterhelfen?'
        });
    }
}
