import { ChannelAccount, StatePropertyAccessor } from 'botbuilder';
import { ComponentDialog, DialogContext, WaterfallDialog, WaterfallStepContext } from 'botbuilder-dialogs';

import { SessionService } from '../../user/session.service';
import { UserService } from '../../user/user.service';
import { UserSettings } from '../user';

const sessionMsg = (url: string, sessionId: string) => `
Mit folgendem Code kannst du dich auf ${url} anmelden:

**${sessionId}**

Achtung, dieser Code ist jetzt nur noch eine Stunde lang gültig.
Brauchst du einen neuen Code oder möchtest dich auf einem neuen Gerät einloggen?
Schreibe mir einfach /login und ich schicke dir einen neuen Login Code.`;

export class StartDialog extends ComponentDialog {

    constructor(
        readonly id: string,
        private readonly userSettings: StatePropertyAccessor<UserSettings>,
        private readonly userService: UserService,
        private readonly sessionService: SessionService) {
        super(id);

        this.addDialog(
            new WaterfallDialog(id + 'wf', [
                this.getSessionId.bind(this)
            ])
        );
    }

    public async onBeginDialog(innerDC: DialogContext, options?: { member: ChannelAccount }) {
        // parse the user's info and save it in the user state
        return await super.onBeginDialog(innerDC, options);
    }

    public async getSessionId(step: WaterfallStepContext) {
        const member: ChannelAccount = (step.options as any).member;
        let [ user ] = await this.userService.find({ telegramId: member.id });
        if (!user) {
            user = await this.userService.create(member.name || '', member.id);
            await step.context.sendActivity('Hallo ' + user.name + ' und herzlich willkommen auf die Eventing Plattform. '
            + 'Das Team wünscht dir viel Spaß beim Browsen und Teilnehmen an Veranstaltungen!');
        }

        await this.userSettings.set(step.context, {
            telegramId: user.telegramId,
            publicId: user.publicId,
            privateId: user.id
        });

        const session = await this.sessionService.create(user);

        let sessionId: string = session.id;
        sessionId = sessionId.substr(0, sessionId.length / 2) + '-' + sessionId.substr(sessionId.length / 2);
        await step.context.sendActivity(sessionMsg(process.env.HOME_URL || '', sessionId));
        return await step.endDialog();
    }
}
