import { Body, Controller, Get, Param, Post, Put, Query, UseGuards } from '@nestjs/common';

import { Session } from '../user/session.entity';
import { UserSession } from '../user/user.decorator';
import { UserSessionGuard } from '../user/user.guard';
import { DateFormater } from '../utility/DateFormater';

import { EventAttendance, EventAttendee } from './attendee.entity';
import { EventsService } from './events.service';

@Controller('/api/events')
export class EventsController {

    constructor(private eventsService: EventsService) { }

    @Post()
    public create(
        @Body('title') title: string,
        @Body('location') location: string,
        @Body('description') description: string,
        @Body('dateStart') dateStart: number,
        @Body('dateEnd') dateEnd: number) {

        const event = this.eventsService.create(title, location, description, dateStart, DateFormater.dateToUnix(new Date()), dateEnd, DateFormater.dateToUnix(new Date()));

        return event;
    }

    @Get(':eventId') public async getOne(
        @Param('eventId') eventId: string,
        @UserSession() session?: Session
    ) {
        const event = await this.eventsService.getEvent(eventId);

        if (session) {
            try {
                const attend = await this.eventsService.getAttendance(eventId, session?.user?.id);

                event.attend = attend.status;

            } catch { }

        }

        return event;
    }

    @Get()
    public async getAll(
        @Query('page') page: number,
        @Query('limit') limit: number,
        @Query() query,
        @UserSession() session?: Session): Promise<any> {
        const res = await this.eventsService.getAll(page, limit, query);

        // send the attendance info with the request
        if (session) {
            try {
                const attendance = await this.eventsService.getManyAttendance(
                    res.results.map((event) => event.id),
                    session.user.id);

                const rsvp = (attendance as any[]).reduce((result, item: EventAttendee) => {
                    const { event, user, ...entity } = item; // extract superfluous data
                    result[item.event.id] = entity;
                    return result;
                }, {});

                return { rsvp, ...res };
            } catch { }

        }

        return res;
    }

    @Put(':eventId/attend/')
    @UseGuards(UserSessionGuard)
    public async attend(
        @Param('eventId') eventId: string,
        @Body('attendance') attendance: EventAttendance,
        @UserSession() session: Session) {
        await this.eventsService.setAttendance(eventId, session.user.id, attendance);
    }

    @Get(':eventId/attend/')
    @UseGuards(UserSessionGuard)
    public async getAttendance(
        @Param('eventId') eventId: string,
        @UserSession() session: Session) {
        await this.eventsService.getAttendance(eventId, session.user.id);
    }
}
