import { Body, Controller, Delete, Get, Headers, Param, Put, Req, UnauthorizedException, UseGuards } from '@nestjs/common';
import { Request } from 'express';

import { Session } from './session.entity';
import { SessionService } from './session.service';
import { UserSession } from './user.decorator';
import { UserSessionGuard } from './user.guard';

@Controller('/api/session')
export class SessionController {

    constructor(private sessionService: SessionService) { }

    @Get()
    @UseGuards(UserSessionGuard)
    public one(@UserSession() session: Session) {
        const parsedSession = {
            ...session,
            user: session.user.publicId
        };
        return parsedSession;
    }

    /**
     * @param sessionId
     * @param userAgent
     * @param req
     */
    @Put()
    public async login(@Body('sessionId') sessionId: string, @Headers('user-agent') userAgent: string, @Req() req: Request) {
        const session = await this.sessionService.one(sessionId);
        if (!this.sessionService.isValid(session) // expired?
            || session.userAgent.length // already logged in somewhere
            || !userAgent.length // empty user agent, quite fishy?
        ) {
            throw new UnauthorizedException();
        }
        session.userAgent = userAgent;

        await this.sessionService.update(session);
        req.session.id = session.id; // set the session cookie
        return session.user;
    }

    @Delete(':id')
    @UseGuards(UserSessionGuard)
    public async remove(@Param('id') id: string, @UserSession() userSess: Session) {
        const session = userSess.id === id ? userSess : await this.sessionService.one(id);
        if (userSess.user.id !== session.user.id) {
            throw new UnauthorizedException();
        }
        await this.sessionService.remove(session);
    }

    // @Get(':uuid')
    // @UseGuards(UserSessionGuard)
    public async list(@UserSession() session: Session) {

        // TODO list all the sessions of that user
        return session;
    }
}
