import { Controller, Delete, forwardRef, Get, Inject, Param, ParseUUIDPipe, Query, Req, UseGuards } from '@nestjs/common';
import { Request } from 'express';
import { In } from 'typeorm';

import { OrganiserService } from '../organiser/organiser.service';

import { Session } from './session.entity';
import { UserSession } from './user.decorator';
import { UserSessionGuard } from './user.guard';
import { UserService } from './user.service';
import { UserMetaService } from './usermeta.service';

@Controller('/api/user')
export class UserController {

    constructor(
        private userService: UserService,
        private userMetaService: UserMetaService,
        @Inject(forwardRef(() => OrganiserService))
        private organiserService: OrganiserService) { }

    @Get()
    @UseGuards(UserSessionGuard)
    public me(@UserSession() session: Session) {
        return session.user;
    }

    @Get('logout')
    public async logout(@Req() req: Request) {
        req.session = null;
        return null;
    }

    @Get(':uuid')
    public async one(@Param('uuid', ParseUUIDPipe) uuid: string) {
        const { id, telegramId, // filter out these keys
            ...user } = await this.userService.one({ publicId: uuid });

        return user;
    }

    @Get('subs')
    @UseGuards(UserSessionGuard)
    public async ownSubscriptions(@UserSession() session: Session, @Query('offset') skip?: 0) {
        return await this.subscriptions(session.user.id, skip);
    }

    @Get(':uuid/subs')
    public async subscriptions(@Param('uuid', ParseUUIDPipe) uuid: string, @Query('offset') skip?: 0) {
        const [orgaIds, total] = await this.userMetaService.getSubscriptions(uuid, skip, 25);
        const entities = await this.organiserService.find({ id: In(orgaIds) });
        return { total, entities };
    }

    @Delete()
    @UseGuards(UserSessionGuard)
    public async remove(@UserSession() session: Session) {
        return await this.userService.remove(session.user);
    }
}
